@@ -4,12 +4,13 @@ use 5.008000;
 use strict;
 use warnings;
 
-our $VERSION = '0.03_01';
+our $VERSION = '0.03_02';
 
 use File::Spec;
 use YAML::XS qw( LoadFile );
 use Cpanel::JSON::XS;
 use Hash::Merge;
+use Scalar::Util qw( refaddr );
 use Carp qw( croak );
 
 my %FILE_EXTENSIONS_MAP = (
@@ -56,6 +57,7 @@ sub new {
   $self->{_hash_merge} = Hash::Merge->new( 'CONFIG_PRECEDENT' );
   $self->{_config}     = undef;
   $self->{_vars}       = {};
+  $self->{_seen_nodes} = {};
 
   return $self;
 }
@@ -83,7 +85,8 @@ sub load {
   $self->{_config} = $self->_build_tree(@config_sections);
   $self->_process_tree( $self->{_config} );
 
-  $self->{_vars} = {};
+  $self->{_vars}       = {};
+  $self->{_seen_nodes} = {};
 
   return $self->{_config};
 }
@@ -183,6 +186,12 @@ sub _process_tree {
 
   $_[0] = $self->_process_node( $_[0] );
 
+  if ( my $node_addr = refaddr( $_[0] ) ) {
+    return if $self->{_seen_nodes}{$node_addr};
+
+    $self->{_seen_nodes}{$node_addr} = 1;
+  }
+
   if ( ref( $_[0] ) eq 'HASH' ) {
     foreach ( values %{ $_[0] } ) {
       $self->_process_tree($_);
@@ -205,7 +214,7 @@ sub _process_node {
 
   if ( !ref($node) && $self->{interpolate_variables} ) {
     $node =~ s/\$((\$?)\{([^\}]*)\})/
-        $2 ? $1 : $self->_resolve_var( $3 )/ge;
+        $2 ? $1 : ( $self->_resolve_var( $3 ) || '' )/ge;
   }
   elsif ( ref($node) eq 'HASH' && $self->{process_directives} ) {
     if ( defined $node->{var} ) {
@@ -259,10 +268,12 @@ sub _resolve_var {
 
   my $vars = $self->{_vars};
 
-  unless ( defined $vars->{$var_name}  ) {
+  unless ( defined $vars->{$var_name} ) {
     my @tokens = split( /\./, $var_name );
     my $pointer = $self->{_config};
 
+    my $value;
+
     while ( 1 ) {
       my $token = shift @tokens;
 
@@ -270,9 +281,8 @@ sub _resolve_var {
         last unless defined $pointer->{$token};
 
         if ( !@tokens ) {
-          $vars->{$var_name}
-              = $self->_process_node( $pointer->{$token} );
-          $pointer->{$token} = $vars->{$var_name};
+          $value = $self->_process_node( $pointer->{$token} );
+          $pointer->{$token} = $value;
 
           last;
         }
@@ -290,9 +300,8 @@ sub _resolve_var {
         last unless defined $pointer->[$token];
 
         if ( !@tokens ) {
-          $vars->{$var_name}
-              = $self->_process_node( $pointer->[$token] );
-          $pointer->[$token] = $vars->{$var_name};
+          $value = $self->_process_node( $pointer->[$token] );
+          $pointer->[$token] = $value;
 
           last;
         }
@@ -302,10 +311,10 @@ sub _resolve_var {
         $pointer = $pointer->[$token];
       }
     }
-  }
 
-  unless ( defined $vars->{$var_name} ) {
-    $vars->{$var_name} = '';
+    if ( defined $value ) {
+      $vars->{$var_name} = $value;
+    }
   }
 
   return $vars->{$var_name};
